package com.compdfkit.conversion.flutter.plugin

import android.content.Context
import android.util.Log
import com.compdfkit.conversion.ConvertError
import com.compdfkit.conversion.flutter.plugin.utils.ConvertUtils
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodChannel
import kotlinx.coroutines.*

/**
 *  ConvertFlutterPlugin.kt
 *
 *  Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 *
 *  THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 *  AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 *  UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 *  This notice may not be removed from this file.
 **/
class ConvertFlutterPlugin : FlutterPlugin, EventChannel.StreamHandler {

    private val channelName = "com.compdfkit.conversion.flutter.convert"


    companion object {

        val PARAMS_CONVERT_TYPE = "convertType"
        val PARAMS_FILE_PATH = "filePath"
        val PARAMS_FILE_NAME = "fileName"
        val PARAMS_TASK_ID = "taskId"
        val PARAMS_CONTAIN_IMAGES = "containImages"
        val PARAMS_CONTAIN_ANNOTATIONS = "containAnnotations"
        val PARAMS_WORK_SHEET_OPTIONS = "workSheetOptions"
        val PARAMS_CONTENTOPTIONS = "contentOptions"
        val PARAMS_IMAGE_DPI = "imageDpi"
        val PARAMS_IMAGE_PAGE_OPTIONS = "imagePageOptions"
        val PARAMS_MERGE_CSV = "mergeCSV"
        val PARAMS_HTML_PAGE_OPTIONS = "htmlPageOptions"

        val TASK_STATUS_WAIT = 0
        val TASK_STATUS_START = 1
        val TASK_STATUS_CONVERTING = 2
        val TASK_STATUS_SUCCESS = 3
        val TASK_STATUS_FAIL = 4
        val TASK_STATUS_CANCEL = 5

    }

    private var convertUtils = ConvertUtils()

    private var context: Context? = null

    override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        context = binding.applicationContext
        val channel = EventChannel(binding.binaryMessenger, channelName)
        channel.setStreamHandler(this)

        val methodChannel = MethodChannel(binding.binaryMessenger, "com.compdfkit.conversion.flutter.convert.method")
        methodChannel.setMethodCallHandler { call, result ->
            when (call.method) {
                "cancel_task" -> {
                    convertUtils.cancel()
                }
                else -> {
                    result.notImplemented()
                }
            }
        }
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {

    }

    private var eventSink: EventChannel.EventSink? = null

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        eventSink = events
        print("CPDFConversion : Android端已收到转档任务，开始进行转档")
        startTask(arguments)
    }

    override fun onCancel(arguments: Any?) {

    }

    private fun startTask(arguments: Any?) {
        GlobalScope.launch(Dispatchers.IO) {
            val convertInfoMap = arguments as Map<String, Any>
            print("CPDFConversion : Android端已收到转档任务，开始进行转档")
            print(convertInfoMap)
            val options = convertUtils.getConvertOptions(convertInfoMap[PARAMS_CONVERT_TYPE]?.toString() ?: "", convertInfoMap)
            val taskId = convertInfoMap[PARAMS_TASK_ID] as? String ?: ""
            if (context == null) {
                return@launch
            }
            val err = convertUtils.onConvert(context = context!!,
                filePath = convertInfoMap[PARAMS_FILE_PATH]?.toString() ?: "",
                convertType = convertInfoMap[PARAMS_CONVERT_TYPE]?.toString() ?: "", pageArrays = null, params = options,
                onHandle = {
                    updateTask(
                        taskId = taskId,
                        progress = 0,
                        taskStatus = TASK_STATUS_START
                    )
                }, onProgress = { current, total ->
                    val progress = ((current / total.toFloat()) * 1000).toInt() / 10
                    Log.e("conversion", "progress:${progress}, taskId:${taskId}")
                    updateTask(
                        taskId = taskId,
                        progress = progress,
                        taskStatus = TASK_STATUS_CONVERTING
                    )
                }, onPost = { code, outFilePath ->
                    Log.e("conversion", "onPost， code:${code.name}, outputPath:${outFilePath}")
                    when (code) {
                        ConvertError.ERR_SUCCESS -> {
                            updateTask(
                                taskId = taskId,
                                progress = 100,
                                taskStatus = TASK_STATUS_SUCCESS,
                                outputPath = outFilePath
                            )
                        }
                        ConvertError.ERR_INTERRUPT ->{
                            updateTask(
                                taskId = taskId,
                                progress = 100,
                                taskStatus = TASK_STATUS_CANCEL
                            )
                        }
                        else -> {
                            updateTask(
                                taskId = taskId,
                                progress = 100,
                                taskStatus = TASK_STATUS_FAIL
                            )
                        }
                    }
                })
        }
    }

    private fun updateTask(taskId: String, progress: Int = 0, taskStatus: Int, outputPath: String? = null) {
        GlobalScope.launch(Dispatchers.Main) {
            eventSink?.success(
                mapOf(
                    "progress" to progress,
                    "taskId" to taskId,
                    "status" to taskStatus,
                    "outputPath" to outputPath
                )
            )
        }
    }
}
